package org.osconsulting.jerr;

import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osconsulting.jerr.errors.JErrConfigError;

/**
 * The <code>JError</code> class is the base class for all <code>JError</code>
 * subclasses classes. The main purpose of this class is to provide a error 
 * code and localized message, based on the user's locale (not just the server's 
 * locale). 
 * 
 * Note that the <code>JError<code> class depends heavily on the 
 * <code>JErrorCode</code> being set. The ErrorCodes are defined in the 
 * implementing <code>JErrorCategory</code> classes. For the JErr there is 
 * one category <code>org.osconsulting.jerr.handler.JErrErrorCategory</code>
 * containing all ErrorCodes</code>.
 * 
 * @author Kurt T Stam
 * Copyright 2006 osConsulting.org Kurt Stam and Shahar Bar
 */
public class JErr extends Exception
{
    private static final long serialVersionUID = 1L;
    private final Log log = LogFactory.getLog(this.getClass());
    private JErrCode jErrorCode;
    String[] args;
 
    /**
     * Gets the error code attached to this error.
     * 
     * @return  the error code.
     */
    public JErrCode getJErrorCode() {
        return jErrorCode;
    }
    /** 
     * Sets the error code.
     * 
     * @param  errorCode - the error code attached to this error.
     */
    public void setJErrorCode(JErrCode jErrorCode) {
        this.jErrorCode = jErrorCode;
    }
    /** Overriding the base class, since the localizedMessage is
     * stored in the message field of this class. There is no API
     * to set the value of localizedMessage of <code>Exception</code>.
     */
    public String getLocalizedMessage()
    {
        return super.getMessage();
    }
    /**
     * Gets the localized error message, based on the locale value
     * passed in. For the search strategy of the localized message
     * see MessageStore.getLocalizedMessage(). Note that a Locale
     * is made up of Language, Country and Variant. 
     * 
     * @param  locale the locale used to find the localized message.
     * 
     * @return the localized error message
     */
    public String getLocalizedMessage(Locale locale) throws JErrConfigError
    {
        return getJErrorCode().getLocalizedMessage(args, locale);
    }
    /**
     * Constructor, taking an error code. The error code is a required argument.
     * 
     * @param  errorCode the errorCode as defined in the implementing 
     *         <code>JErrorCategory</code> class to which this errorCode 
     *         belongs (required argument).
     */
    public JErr(JErrCode jErrorCode) {
        super();
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
    }
    /**
     * Constructor, taking an error code. The error code is a required argument.
     * 
     * In addition an array of strings can be passed in for string
     * replacement in message. For example "{0}" will be replaced
     * by the first argument in the array, "{1}" with the second
     * argument. The arguments may be filenames etc. Make sure to 
     * pass in locale insensive strings.
     * 
     * @param  errorCode the errorCode as defined in the implementing 
     *         <code>JErrorCategory</code> class to which this errorCode 
     *         belongs (required argument).
     * @param  args argument array which will be used to replace variables.
     */
    public JErr(JErrCode jErrorCode, String[] args) {
        super();
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
        this.args = args;
    }
    /**
     * Constructor. Note that the error code is a required argument.
     * 
     * @param  errorCode  the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  localizedMessage error message localized to the locale of the server. 
     * @param  cause the exception that caused this error
     */
    public JErr(JErrCode jErrorCode, String[] args, String localizedMessage, Throwable cause) {
        super(localizedMessage, cause);
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
        this.args = args;
    }
    /**
     * Constructor. Note that the error code is a required argument.
     * 
     * @param  errorCode  the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  localizedMessage error message localized to the locale of the server. 
     * @param  cause the exception that caused this error
     */
    public JErr(JErrCode jErrorCode, String localizedMessage, Throwable cause) {
        super(localizedMessage, cause);
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
    }
    /**
     * Constructor.
     * 
     * @param  errorCode  the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  localizedMessage the error message in the server's locale.
     */
    public JErr(JErrCode jErrorCode, String[] args, String localizedMessage) {
        super(localizedMessage);
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
        this.args = args;
    }
    /**
     * Constructor.
     * 
     * @param  errorCode  the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  localizedMessage the error message in the server's locale.
     */
    public JErr(JErrCode jErrorCode, String localizedMessage) {
        super(localizedMessage);
        if (jErrorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = jErrorCode;
        }
    }
    /**
     * Constructor.
     * 
     * @param  errorCode the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  cause the exception that caused this exception.
     */
    public JErr(JErrCode errorCode, String[] args, Throwable cause) {
        super(cause);
        if (errorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = errorCode;
        }
        this.args = args;
    }
    /**
     * Constructor.
     * 
     * @param  errorCode the errorCode as defined in the implementing JErrorCategory class
     *         to which this errorCode belongs (required argument).
     * @param  cause the exception that caused this exception.
     */
    public JErr(JErrCode errorCode, Throwable cause) {
        super(cause);
        if (errorCode == null) {
            log.debug("JErrorCode is null, defaulting to UE001");
            this.jErrorCode = JErrCategory.UE001_UNSPECIFIED_ERROR;
        } else {
            this.jErrorCode = errorCode;
        }
    }
    /**
     * @see <code>String.toString()</code>
     */
    public String toString() {
        return "ErrorCode=[" + this.jErrorCode + "] Message=[" + getLocalizedMessage() + "]";
    }
}
