package org.osconsulting.l10n;

import java.io.Serializable;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osconsulting.jerr.errors.JErrConfigError;
import org.osconsulting.l10n.msgstore.MessageStore;
/**
 * An JErrorCode contains alphaumeric code and belongs to a certain category of
 * errors. All JErrorCodes need to be listed in an implementation of the JErrorCategory 
 * abstact class. 
 * 
 * @author Kurt Stam
 * Copyright 2006 osConsulting.org Kurt Stam and Shahar Bar
 *
 */
public class LocalizedMessage implements Serializable
{
    private static final long serialVersionUID = 1L;
    private String key;
    private transient Collection<MessageStore> messageStores;
    private transient Log log = LogFactory.getLog(this.getClass());
    /**
     * Constructor, should only be called from the custom JErrorCategory to which
     * this errorCode belongs. The error code is the key into error MessageStore to 
     * find the localized error message in the MessageStore.
     * 
     * @param code - alpha numeric code, can be used to display or to lookup the 
     * localized error message.
     * @param errorCategoryClass - the custom implementation of the JErrorCategory class to 
     * which this errorCode belongs.
     */
    public LocalizedMessage(String key, Collection<MessageStore> messageStores){
        this.key           = key;
        this.messageStores = messageStores;
    }
    /**
     * Gets the alphanumeric error code.
     * @return - the code of the errorCode
     */
    public String getKey() {
        return key;
    }
    /**
     * @see String.toString()
     */
    public String toString() {
        return "\n Code=" + this.key;
    }
    
    public String getLocalizedMessage(Locale locale)
    {
        return getLocalizedMessage(null, locale);
    }
    
    public String getLocalizedMessage(String[] args, Locale locale)
    {
        if (log.isDebugEnabled()) {
            log.debug("Looking up localized message args=[" + args + "] for locale=" + locale);
        }
        String localizedMessage = null;
        if (messageStores!=null && messageStores.size()>0) {
            for (MessageStore messageStore: messageStores) {
                try {
                    if (log.isDebugEnabled()) {
                        log.debug("Looking in MessageStore=" + messageStore.getClass());
                    }
                    localizedMessage = messageStore.getLocalizedMessage(key, locale);
                    if (log.isDebugEnabled()) {
                        log.debug("message=" + localizedMessage);
                    }
                } catch (JErrConfigError jerr) {
                    if (jerr.getLocalizedMessage()!=null) {
                        log.error(jerr.getLocalizedMessage(), jerr);
                    } else {
                        log.error("JERR001 : Could not connect to any message stores");
                    }
                }
                if (localizedMessage != null) break;
            }
            // merge the arguments into the message if there are any
            if (localizedMessage!=null && args!=null) {
                MessageFormat mf = new MessageFormat(localizedMessage);
                return mf.format(args);
            }
        } else {
            log.error("No message stores defined");
        }
        return localizedMessage;
    }

}
