package org.osconsulting.l10n.msgstore;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The configuration manager which reads the jerr-config.xml. JErr
 * assumes that there is one and only one jerr-config.xml which
 * is common for the system. In it, for each error category one can 
 * specify one or more message stores from which is can read the 
 * locale specific error message, for an ErrorCode.
 * 
 * @author kstam
 * Copyright 2006 osConsulting.org Kurt Stam and Shahar Bar
 */
public class ConfigurationManager
{
    private static final Log log = LogFactory.getLog(ConfigurationManager.class);
    private static ConfigurationManager configurationManagerInstance;
    private static XMLConfiguration jerrConfig;
    private static String JERR_CONFIG = "jerr-config.xml";
    //private static String DEFAULT = "default-";
    
    private ConfigurationManager() throws ConfigurationException{
        String configUrl = getConfigUrl();
        log.info("Reading JErr config file " + configUrl);
        XMLConfiguration.setDelimiter('|');
        jerrConfig = new XMLConfiguration(configUrl);
        jerrConfig.setReloadingStrategy(new FileChangedReloadingStrategy());
    }
    
    public static ConfigurationManager getInstance()
    {
        try {
            if (configurationManagerInstance==null){
                configurationManagerInstance = new ConfigurationManager();
            }
            return configurationManagerInstance;
        } catch (ConfigurationException e) {
            //TODO
            return null;
        }
    }
    
    public List<MessageStore> getMessageStores(Class messageCategory)
    {
        List<MessageStore> messageStores = new ArrayList<MessageStore>();
        
        List classes = jerrConfig.getList("categories.category.class");
        int categoryCount = 0;
        for (Iterator i=classes.iterator();i.hasNext();) {
            String className = (String) i.next();
            if (messageCategory.getName().equals(className)) {
                log.debug("Reading configuration for category " + messageCategory.getName());
                List storeClasses= jerrConfig.getList("categories.category(" + categoryCount + ")"
                        + ".message-stores.message-store.class");
                int storeCount = 0;
                for (Iterator j=storeClasses.iterator(); j.hasNext();) {
                    String storeClass = (String) j.next();
                    log.info("Reading info for class:" + storeClass);
                    List names = jerrConfig.getList("categories.category(" 
                            + categoryCount + ").message-stores.message-store(" 
                            + storeCount + ").properties.property[@name]");
                    int nameCount=0;
                    HashMap<String,String> properties = new HashMap<String,String>();
                    for (Iterator k=names.iterator(); k.hasNext();) {
                        String name  =(String) k.next();
                        String value = (String) jerrConfig.getProperty("categories.category(" 
                                + categoryCount + ").message-stores.message-store(" 
                                + storeCount + ").properties.property(" + nameCount + ")[@value]");
                        log.info("name=" + name + " value=" + value);
                        properties.put(name,value);
                        nameCount++;
                    }
                    try {
                        Class messageStoreClass = Class.forName(storeClass);
                        Constructor constructor = messageStoreClass.getConstructor(new Class[]{HashMap.class});
                        MessageStore messageStore = (MessageStore)constructor.newInstance(new Object[]{properties});
                        messageStores.add(messageStore);
                    } catch (Exception e) {
                        log.error(e.getLocalizedMessage(), e);
                    }
                    storeCount++;
                }
            }
            categoryCount++;
        }
        
        return messageStores;
    }
    /**
     * Search strategy for the jerr-config file.
     * @return
     */
    private String getConfigUrl()
    {
        String jerrConfigUrl=JERR_CONFIG;
        if (System.getProperty("jerr.config.url") != null) {
            jerrConfigUrl = System.getProperty("jerr.config.url") + JERR_CONFIG;
            log.debug("Using jerr.config.url: " + jerrConfigUrl);
        } else if (System.getProperty("jboss.server.config.url") != null) {
            jerrConfigUrl = System.getProperty("jboss.server.config.url") + JERR_CONFIG;
             log.debug("Using jboss.server.config.url: " + jerrConfigUrl);
        } else if (System.getProperty("catalina.home") != null) {
            jerrConfigUrl = System.getProperty("catalina.home") + "/bin/" + JERR_CONFIG;
            log.debug("Using catalina.home: " + jerrConfigUrl);
        }
        //TODO Check if the file can be found, if not use the default one
        //jerrCOnfigUrl = DEFAULT + JERR_CONFIG;
        return jerrConfigUrl;
    }    
}
