package org.osconsulting.l10n.msgstore;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Locale;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osconsulting.jerr.JErrCategory;
import org.osconsulting.jerr.errors.JErrConfigError;


public class DBBasedMessageStore implements MessageStore
{
	private final Log log = LogFactory.getLog(this.getClass());
    public static final String CONNECTION_URL  = "connectionURL";
    public static final String CONNECTION_NAME = "connectionName";
    public static final String CONNECTION_PWD  = "connectionPassword";
    public static final String DRIVER_NAME     = "driverName";
    public static final String MSG_QUERY_STR   = "messageQueryString";
    public static final String DS_JNDI_NAME    = "dsJndiName";
	
	private final static String SQL_MESSAGE_COLUMN 	= "message";
	// private final static String SQL_LOCALE_COLUMN 	= "locale";
	// private final static String SQL_CODE_COLUMN 	= "code";	
	
	private Connection connection;
	
	private String messageQuery;	
	
    /** (non-Javadoc)
     * @see main.java.org.osconsulting.jerr.handler.MessageStore#getLocalizedMessage(main.java.org.osconsulting.jerr.JErrorCode, java.util.Locale)
     */
    public String getLocalizedMessage(String key, Locale locale) throws JErrConfigError {
    	String message = null;
    	
    	try {
    		Collection<String> localeStrings = getLocaleStrings(locale);
    		for (String localeString:localeStrings) {
		        PreparedStatement stmnt = connection.prepareStatement(messageQuery);		        
		        stmnt.setString(1, localeString);
		        stmnt.setString(2, key);
		        if (stmnt.execute()) {
		        	ResultSet rs = stmnt.getResultSet();
		        	if (rs.next()) {
		        		message = rs.getString(SQL_MESSAGE_COLUMN);
		        		break;
		        	}
		        }
		        stmnt.close();
    		}    		
    	}
    	catch(SQLException sqlEx) {
    		throw new JErrConfigError(JErrCategory.JE005_INVALID_MESSAGEQRY, sqlEx.getLocalizedMessage(), sqlEx);
    	}
    	
    	return message;
    }
    
    /**
     * @param configProperties Property file containing database connection parameters
     * @throws JErrConfigError
     */
    public DBBasedMessageStore(HashMap<String,String> properties) throws JErrConfigError 
    {
        messageQuery  = (String) properties.get(MSG_QUERY_STR);
        String dsJndiName = (String) properties.get(DS_JNDI_NAME);
        if (dsJndiName != null) {
            try {
                Context context = new InitialContext();
                DataSource ds = (DataSource) context.lookup(dsJndiName);
                connection = ds.getConnection();
            } catch (NamingException ne) {
                throw new JErrConfigError(JErrCategory.JE009_INVALID_JNDI_REF, dsJndiName, ne.getLocalizedMessage(), ne);
            } catch (SQLException se) {
                throw new JErrConfigError(JErrCategory.JE006_DB_CONNECTION_ERROR, se.getLocalizedMessage(), se);
            }
        } else {
            
	        String url 		= (String) properties.get(CONNECTION_URL);
	        String user 	= (String) properties.get(CONNECTION_NAME);
	        String pw 		= (String) properties.get(CONNECTION_PWD);
	        String driver 	= (String) properties.get(DRIVER_NAME);
            try {
                // Load JDBC driver
                Class.forName(driver);
                connection = DriverManager.getConnection(url, user, pw);
                log.info("Succesfully opened connection to: " + url);
            } catch (ClassNotFoundException cnfe) {
                throw new JErrConfigError(JErrCategory.JE007_INVALID_JDBC_DRIVER, cnfe.getLocalizedMessage(), cnfe);
            } catch (SQLException se) {
                throw new JErrConfigError(JErrCategory.JE006_DB_CONNECTION_ERROR, se.getLocalizedMessage(), se);
            }
        }
    }
    
    /**
     * @param locale
     * @return Collection of Locale Strings based on hierarchy, from most specific 
     * to most generic.
     * @throws Exception
     */
    private Collection<String> getLocaleStrings(Locale locale) {   
    	Collection<String> localeAL = new ArrayList<String>();
    	
    	// Use default Locale if it isnt specified
    	if (locale == null)
    		locale = Locale.getDefault();
    	
    	String language = locale.getLanguage();
		String country 	= locale.getCountry();
		String variant 	= locale.getVariant();		
				
		if (language != null && country != null && variant != null) {
			localeAL.add(language + "_" + country + "_" + variant);
			localeAL.add(language + "_" + country);
			localeAL.add(language);
		}
		else if (language != null && country != null) {
			localeAL.add(language + "_" + country);
			localeAL.add(language);
		}
		else if (language != null) {
			localeAL.add(language);
		}
		
		return localeAL;		
    }

}
